// Ionic Starter App

// angular.module is a global place for creating, registering and retrieving Angular modules
// 'starter' is the name of this angular module example (also set in a <body> attribute in index.html)
// the 2nd parameter is an array of 'requires'
// 'starter.controllers' is found in controllers.js
angular.module('starter', ['ionic', 'ngCordova', 'ptv.services', 'ptv.controllers', 'ptv.directives'])

.constant('apiUrl', '/api')

.constant('ApiEndpoint', {
    url: "https://www.avl.gva.es/apidnv2/rest"
})

.constant('$ionicLoadingConfig', {
    delay: 1,
    duration: 30000,
    template: '<ion-spinner></ion-spinner>'
})

.run(function($ionicPlatform, $rootScope, $state) {
    $ionicPlatform.ready(function() {
        if (ionic.Platform.isIOS()) {
            ionic.Platform.fullScreen();
        }
        if (window.cordova && window.cordova.plugins.Keyboard) {
            cordova.plugins.Keyboard.hideKeyboardAccessoryBar(true);
            cordova.plugins.Keyboard.disableScroll(true);
        }
    });

    $rootScope.$on("$stateChangeStart", function(event, toState) {
        if (toState.isHome) {
            $rootScope.isHome = true;
        } else {
            $rootScope.isHome = false;
        }
    });

    $state.go("app.home");
})

.config(function($stateProvider, $urlRouterProvider, $ionicConfigProvider) {

    $ionicConfigProvider.backButton.previousTitleText(false).text('&emsp;');
    $ionicConfigProvider.navBar.alignTitle('center');
    $ionicConfigProvider.views.swipeBackEnabled(false);
    if (!ionic.Platform.isIOS()) {
        $ionicConfigProvider.scrolling.jsScrolling(false);
    }

    $stateProvider
        .state('app', {
            url: '/app',
            abstract: true,
            templateUrl: 'templates/menu.html',
            controller: 'MenuController'
        })

    .state('app.home', {
        cache: false,
        url: '/home',
        views: {
            'menuContent': {
                templateUrl: 'templates/home.html',
                controller: 'HomeController'
            }
        },
        isHome: true
    })

    .state('app.search-results', {
        url: '/search-results',
        params: {
            word: null
        },
        views: {
            'menuContent': {
                templateUrl: 'templates/search-results.html',
                controller: 'SearchResultsController'
            }
        }
    })

    .state('app.simple-search', {
        cache: false,
        url: '/simple-search',
        params: {
            term: null,
            in_exemple: null,
            in_definicio: null,
            fromAdvance: false
        },
        views: {
            'menuContent': {
                templateUrl: 'templates/simple-search.html',
                controller: 'SimpleSearchController'
            }
        }
    })

    .state('app.advanced-search', {
        url: '/advanced-search',
        params: {
            word: null,
            in_exemple: null,
            in_definition: null
        },
        views: {
            'menuContent': {
                templateUrl: 'templates/advanced-search.html',
                controller: 'AdvancedSearchController'
            }
        }
    })

    .state('app.advanced-search-results', {
        url: '/advanced-search-results',
        params: { data: null },
        views: {
            'menuContent': {
                templateUrl: 'templates/advanced-search-results.html',
                controller: 'AdvancedSearchResultsController'
            }
        }
    });

    // if none of the above states are matched, use this as the fallback
    //$urlRouterProvider.otherwise('/app/home');
});

function handleOpenURL(url) {
    console.log("Handle Open Url: " + url);
    var wordToSearch = getParameterByName("search", url);
    if (typeof wordToSearch != "undefined" && wordToSearch != null && wordToSearch != "") {
        console.log("Word To Search: " + wordToSearch);
        var body = document.getElementsByTagName("body")[0];
        var appLaunchedController = angular.element(body).scope();
        appLaunchedController.loadViaURL(decodeURIComponent(wordToSearch));
    }
}

function getParameterByName(name, url) {
    if (!url) {
        url = window.location.href;
    }
    name = name.replace(/[\[\]]/g, "\\$&");
    var regex = new RegExp("[?&]" + name + "(=([^&#]*)|&|#|$)"),
        results = regex.exec(url);
    if (!results) return null;
    if (!results[2]) return '';
    return decodeURIComponent(results[2].replace(/\+/g, " "));
}