angular.module('ptv.controllers', [])

.controller('MenuController', function($scope, $ionicModal, $rootScope, $cordovaAppVersion) {
    $ionicModal.fromTemplateUrl('templates/introduccio.html', { scope: $scope }).then(function(modal) { $scope.introduccioModal = modal; });
    $ionicModal.fromTemplateUrl('templates/criteris.html', { scope: $scope }).then(function(modal) { $scope.criterisModal = modal; });
    $ionicModal.fromTemplateUrl('templates/abreviatures.html', { scope: $scope }).then(function(modal) { $scope.abreviaturesModal = modal; });
    $ionicModal.fromTemplateUrl('templates/ajuda.html', { scope: $scope }).then(function(modal) { $scope.ajudaModal = modal; });
    $ionicModal.fromTemplateUrl('templates/propostes.html', { scope: $scope }).then(function(modal) { $scope.propostesModal = modal; });

    document.addEventListener("deviceready", function() {
        $cordovaAppVersion.getVersionNumber().then(function(version) {
            var date = new Date();
            $scope.appVersion = {
                year: date.getFullYear(),
                version: version
            }
        });
    }, false);

})

.controller('ModalPropostesController', function($scope, $ionicScrollDelegate, $ionicPopup, $ionicLoading, api) {

    $scope.$on('modal.shown', function() {
        if ($scope.propostesModal.isShown()) {
            $scope.data = {};
            $ionicScrollDelegate.$getByHandle('modalPropostesContent').scrollTop(true);
        }
    });

    $scope.submit = function() {
        var word = $scope.data.word;
        var definition = $scope.data.definition;
        var name = $scope.data.name;
        var lastname = $scope.data.lastname;
        var email = $scope.data.email;
        var accept = $scope.data.accept;

        if (!isNotEmpty(word) || !isNotEmpty(definition) || !isNotEmpty(name) || !isNotEmpty(lastname) || !isNotEmpty(email) || !isNotEmpty(accept)) {
            var alertPopup = $ionicPopup.alert({
                title: "Falten dades",
                template: "Ha d'omplir els camps obligatoris (*) i acceptar la política de privacitat.",
                buttons: [{ text: "Tancar" }]
            });
        } else {
            $ionicLoading.show();

            var data = {
                nom: name,
                cognom: lastname,
                adressa: email,
                paraula: word,
                proposta: definition,
                dnv: false
            }

            if(isNotEmpty($scope.data.dni)) data.dni = $scope.data.dni;
            if(isNotEmpty($scope.data.phone)) data.telefon = $scope.data.phone;
            if(isNotEmpty($scope.data.job)) data.professio = $scope.data.job;
            if(isNotEmpty($scope.data.city)) data.localitat = $scope.data.city;
            if(isNotEmpty($scope.data.birthPlace)) data.llocNaiximent = $scope.data.birthPlace;
            if(isNotEmpty($scope.data.birthYear)) data.anyNaiximent = $scope.data.birthYear;
            if(isNotEmpty($scope.data.examples)) data.exemples = $scope.data.examples;
            if(isNotEmpty($scope.data.documentation)) data.documentacio = $scope.data.documentation;

            api.proposal(data).then(function(result) {
                $ionicLoading.hide();
                if (result.data != null && typeof result.data != "undefined" && result.data.result) {
                    var alertPopup = $ionicPopup.alert({
                        title: "Proposta enviada",
                        template: "Gràcies per col·laborar amb l'Acadèmia Valenciana de la Llengua.",
                        buttons: [{ text: "Tancar" }]
                    });
                    $scope.data = {};
                    $ionicScrollDelegate.$getByHandle('modalPropostesContent').scrollTop(true);
                } else {
                    var alertPopup = $ionicPopup.alert({
                        title: "Error",
                        template: "No s'ha pogut enviar la seua proposta. Prove més tard.",
                        buttons: [{ text: "Tancar" }]
                    });
                }
            }, function(error) {
                $ionicLoading.hide();
                var alertPopup = $ionicPopup.alert({
                    title: "Error",
                    template: "No s'ha pogut enviar la seua proposta. Prove més tard.",
                    buttons: [{ text: "Tancar" }]
                });
            });
        }

    }

})

.controller('StartAppController', function($scope, $state, $ionicHistory, goToSearch) {
    $scope.loadViaURL = function(word) {
        $ionicHistory.clearCache().then(function(){
            $ionicHistory.clearHistory();
            $ionicHistory.nextViewOptions({
                disableBack: true
            });
            goToSearch.search = true;
            goToSearch.word = word;
            $state.go("app.home", {}, {
                reload: true,
                notify: true
            });
        });
    }
})

.controller('HomeController', function($state, $scope, $rootScope, $http, $ionicPopup, $ionicLoading, $ionicHistory, $ionicPlatform, dropDowns, goToSearch) {

    $scope.$on('$ionicView.enter', function(){
        $ionicHistory.clearCache();
    });

    $scope.data = {};
    $rootScope.advancedSearch = null;

    if (goToSearch.search) {
        goToSearch.search = false;
        $ionicHistory.clearCache().then(function() {
            $ionicHistory.clearHistory();
            $rootScope.forceSearch = true;
            $state.go("app.search-results", { word: goToSearch.word, random: Math.floor((Math.random()*6)+1) }, { reload: true, cache: false, notify: true });
        });
    } else {
        if (ionic.Platform.isIOS()) {
            //Hack for keyboard hides
            window.addEventListener('native.hidekeyboard', function() {
                $ionicLoading.show({ duration: 1, template: "<div></div>" });
            });
        }

        $scope.submit = function() {
            if (typeof $scope.data.word == "undefined" || $scope.data.word == null || $scope.data.word == "") {
                var alertPopup = $ionicPopup.alert({
                    title: "Busca buida",
                    template: "Ha d'escriure un text per a poder fer una busca.",
                    buttons: [{ text: "Tancar" }]
                });
            } else {
                $rootScope.forceSearch = true;
                $state.go("app.search-results", { word: $scope.data.word });
            }
        };

        $scope.goAdvanced = function() {
            $ionicLoading.show();
            if (!$rootScope.infoDropDowns.isLoaded) {
                dropDowns.load().then(function(result) {
                    $ionicLoading.hide();
                    $state.go("app.advanced-search", { word: $scope.data.word });
                }, function(error) {
                    $ionicLoading.hide();
                    $state.go("app.advanced-search", { word: $scope.data.word });
                });
            } else {
                $ionicLoading.hide();
                $state.go("app.advanced-search", { word: $scope.data.word });
            }
        }

        $ionicPlatform.ready(function() {
            if (typeof navigator != "undefined" && typeof navigator.splashscreen != "undefined") {
                navigator.splashscreen.hide();
            }
        });
    }
})

.controller('SearchResultsController', function($scope, $rootScope, $state, $stateParams, $ionicLoading, api, searchBack, openSibling) {

    $scope.$on('$ionicView.enter', function(){
        if($rootScope.forceSearch) {
            $rootScope.forceSearch = false;
            $ionicLoading.show();
            $scope.sibling = { openNow:  true };
            $scope.searchingStatus = 0;
            $scope.data = { hasInfo: false, openConjugation: false, onDNV: false };

            $scope.word = $stateParams.word;
            if (searchBack.newSearch)  {
                $scope.word = searchBack.word;
                searchBack.newSearch = false;
                searchBack.word = null;
            }

            api.simpleSearch($scope.word).then(function(result) {
                if (result.data != null && result.data.length > 0) {
                    $scope.items = result.data;
                    $scope.searchingStatus = 2;
                    $ionicLoading.hide();
                    if($scope.items.length == 1) openDefinition($scope.items[0]);
                } else {
                    searchOnSibling($scope.word);
                }
            }, function(error) {
                $scope.searchingStatus = 1;
                $ionicLoading.hide();
            });
        }
    });

    $scope.openDefinition = function(term) {
        openDefinition(term);
    }

    $scope.openSiblingApp = function($event) {
        openSiblingApp();
    }

    function openDefinition(term) {
        api.log({
            busqueda: $scope.word,
            navegador: ionic.Platform.platform(),
            filtres: {},
            seleccionada: term.lema
        });
        $state.go("app.simple-search", {
            term: term
        });
    }

    function searchOnSibling()  {
        api.interconexio($scope.word).then(function(result) {
            if (result.data != null && result.data) {
                $scope.onDNV = true;
                $scope.searchingStatus = 1;
                $ionicLoading.hide();
                var openInSiblingNow = localStorage.getItem("openSiblingNow");
                if (openInSiblingNow != "undefined" && openInSiblingNow) {
                    $scope.data.hideOpenRemember = true;
                    openSiblingApp();
                }
            } else {
                searchByLevenshtein();
            }
        }, function(error) {
            $scope.searchingStatus = 1;
            $ionicLoading.hide();
        });
    }

    function searchByLevenshtein()  {
        api.simpleSearchLevenshtein($scope.word).then(function(result) {
            if (result.data != null && result.data.length > 0) {
                $scope.onDNV = true;
                var openInSiblingNow = localStorage.getItem("openSiblingNow");
                if (openInSiblingNow != "undefined" && openInSiblingNow) {
                    $scope.data.hideOpenRemember = true;
                    openSiblingApp();
                }
            }
            $scope.searchingStatus = 1;
            $ionicLoading.hide();
        }, function(error) {
            $scope.searchingStatus = 1;
            $ionicLoading.hide();
        });
    }

    function openSiblingApp() {
        if($scope.sibling.openNow !== "undefined" && $scope.sibling.openNow) {
            $scope.data.hideOpenRemember = true;
            localStorage.setItem("openSiblingNow", true);
        }
        openSibling.openWord($scope.word);
    }
})

.controller('SimpleSearchController', function($scope, $state, $rootScope, $ionicPopup, $stateParams, $cordovaSocialSharing, $ionicLoading, $ionicScrollDelegate, $timeout, $ionicHistory, api, searchBack, historial, goToSearch, openSibling) {

    $scope.navbarTitle = "Definició";
    $scope.data = { hasInfo: false, openConjugation: false };
    $scope.historyPosition = historial.retrieve().length - 1;
    $scope.sibling = { openNow:  true };

    $scope.resizeLemaTitle = function() {
        $timeout(function(){
            var dl = jQuery(".lema-title");
            if(dl.length > 0) dl.css("min-height", dl.outerHeight());
        });
    }

    $scope.submit = function() {
        if (typeof $scope.data.word == "undefined" || $scope.data.word == null || $scope.data.word == "") {
            $ionicPopup.alert({
                title: "Busca buida",
                template: "Ha d'escriure un text per a poder fer una busca.",
                buttons: [{ text: "Tancar" }]
            });
        } else {
            if (typeof cordova != "undefined") cordova.plugins.Keyboard.close(); //only available on mobile device
            if (!$stateParams.fromAdvance) {
                searchBack.newSearch = true;
                searchBack.word = $scope.data.word;
                $rootScope.forceSearch=true;
                $ionicHistory.clearCache().then(function() {
                    $ionicHistory.goBack();
                });
            } else {
                $ionicHistory.clearCache().then(function() {
                    $ionicHistory.nextViewOptions({
                        disableBack: true
                    });
                    goToSearch.search = true;
                    goToSearch.word = $scope.data.word;
                    $state.go("app.home");
                    $ionicHistory.clearHistory();
                });
            }
        }
    }

    if ($stateParams.term != null && typeof $stateParams.term != "undefined") {
        $scope.data.word = $stateParams.term.lema;
        $scope.data.in_exemple = $stateParams.in_exemple;
        $scope.data.original_term = $stateParams.term;
        $scope.data.in_definicio = $stateParams.in_definicio;
        historial.add($scope.data.word);
        $scope.historyPosition = historial.retrieve().length - 1;
        processLema($stateParams.term);
        $scope.data.lema = $stateParams.term;
        loadDefinitions($scope.data.lema.pkey);
    }

    $scope.stateBack = function() {
        $ionicHistory.goBack();
    };

    $scope.share = function($event) {
        $cordovaSocialSharing.share(
            "Consulta la definició de: " + $scope.data.lema.lema,
            "PTV - AVL",
            null,
            "http://www.avl.gva.es/lexicval/ptv?paraula=" + encodeURIComponent($scope.data.lema.lema)
        );
    }

    $scope.openSiblingApp = function($event) {
        openSiblingApp();
    }

    $scope.gotoRem = function($event, pkey, nivell, acc) {
        $event.preventDefault();
        $ionicLoading.show();
        $scope.data = { hasInfo: false, openConjugation: false };
        api.wordByDefinition(pkey).then(function(result) {
            if (result.data != null && typeof result.data != "undefined" && result.data != "") {
                $scope.data.word = result.data.lema;
                $scope.data.defSel = acc.split(";");
                historial.add($scope.data.word);
                $scope.historyPosition = historial.retrieve().length - 1;
                processLema(result.data);
                $scope.data.lema = result.data;
                loadDefinitions($scope.data.lema.pkey);
            } else {
                api.wordByDefinitionDNV(pkey).then(function(result) {
                    if (result.data != null && typeof result.data != "undefined") {
                        $scope.data.onDNV = true;
                        $scope.data.dnvPkey = pkey;
                        $scope.data.dnvAcc = acc;
                        $ionicLoading.hide();
                        var openInSiblingNow = localStorage.getItem("openSiblingNow");
                        if (openInSiblingNow != "undefined" && openInSiblingNow) {
                            $scope.data.hideOpenRemember = true;
                            openSiblingApp();
                        }
                    } else {
                        $ionicLoading.hide();
                    }
                }, function(error) {
                    $ionicLoading.hide();
                });
            }
        }, function(error) {
            $ionicLoading.hide();
        });
    }

    $scope.allDefinitions = function() {
        $ionicLoading.show();
        $scope.data.defSel = null;
        $scope.data.in_exemple = null;
        $scope.data.in_definicio = null;
        loadDefinitions($scope.data.lema.pkey);
        $ionicLoading.hide();
    }

    $scope.openDefinition = function() {
        $scope.navbarTitle = "Definició";
        $ionicLoading.show();
        $scope.data.openConjugation = false;
        $ionicLoading.hide();
    }

    $scope.openConjugation = function() {
        $scope.navbarTitle = "Conjugació";
        $ionicLoading.show();
        $scope.data.openConjugation = true;
        $ionicLoading.hide();
    }

    $scope.backHistory = function($event) {
        $event.preventDefault();
        var hist = historial.retrieve();
        $scope.historyPosition = $scope.historyPosition - 1;
        search(hist[$scope.historyPosition]);
    }

    $scope.nextHistory = function($event) {
        $event.preventDefault();
        var hist = historial.retrieve();
        $scope.historyPosition = $scope.historyPosition + 1;
        search(hist[$scope.historyPosition]);
    }

    function search(word) {
        $ionicLoading.show();
        $scope.data = { hasInfo: false, openConjugation: false };
        $scope.data.word = word;
        api.simpleSearch($scope.data.word).then(function(result) {
            if (result.data != null && result.data.length > 0) {
                processLema(result.data[0]);
                $scope.data.lema = result.data[0];
                loadDefinitions($scope.data.lema.pkey);
            } else {
                $ionicLoading.hide();
            }
        }, function(error) {
            $ionicLoading.hide();
        });
    }

    function processLema(element) {
        var detalllema = jQuery("<div>" + element.detalllema + "</div>");
        var infconf = detalllema.find(".infconf");
        element.infconf = null;
        if (infconf.length > 0) {
            infconf = infconf[0].outerHTML;
            element.infconf = infconf;
        }
        detalllema.find(".infconf").remove();
        element.detalllema = detalllema[0].outerHTML;

        if (element.conjugacioApp != null && element.conjugacioApp != '') {
            var conjugacioApp = jQuery("<div>" + element.conjugacioApp + "</div>");
            conjugacioApp.find("br").remove();
            element.conjugacioApp = conjugacioApp[0].outerHTML;
        }
    }

    function loadDefinitions(pkey) {

        var definicionsData = {};
        if (typeof $scope.data.in_exemple != "undefined" && $scope.data.in_exemple != null && $scope.data.in_exemple != "") {
            definicionsData.exemple = $scope.data.in_exemple;
        }
        if (typeof $scope.data.in_definicio != "undefined" && $scope.data.in_definicio != null && $scope.data.in_definicio != "") {
            definicionsData.definicio = $scope.data.in_definicio;
        }

        api.definitions(pkey, definicionsData).then(function(result) {
            processDefinitions(result);
        }, function(error) {
            $ionicLoading.hide();
        });
    }

    function processDefinitions(result) {
        if (typeof result.data != "undefined" || result.data.length > 0) {
            for (var i = 0; i < result.data.length; i++) {
                var def = result.data[i];
                def.definiciohtml = "<div>" + def.definiciohtml.replace("onclick='gotoRem(", "ng-click='gotoRem($event, ") + "</div>";
                if (typeof $scope.data.in_definicio != "undefined" && $scope.data.in_definicio != null && $scope.data.in_definicio != "") {
                    def.definiciohtml = jQuery(def.definiciohtml).find(".descripcio .definicio, .descripcio .definicioDes, .descripcio .locucio")
                        .highlight($scope.data.in_definicio, { wordsOnly: true })
                        .prevObject[0].innerHTML;
                }
                if (typeof $scope.data.in_exemple != "undefined" && $scope.data.in_exemple != null && $scope.data.in_exemple != "") {
                    def.definiciohtml = jQuery(def.definiciohtml).find(".descripcio .exemples")
                        .highlight($scope.data.in_exemple, { wordsOnly: true })
                        .prevObject[0].innerHTML;
                }
            }
            $scope.data.definicions = result.data;
            $scope.data.hasInfo = true;
            checkHistoryButton();
            $ionicLoading.hide();
        } else {
            $ionicLoading.hide();
        }
    }

    function checkHistoryButton() {
        var hist = historial.retrieve();
        if (hist.length > 1) {
            if ($scope.historyPosition > 0) $scope.backHistoryButton = true;
            else $scope.backHistoryButton = false;

            if ($scope.historyPosition < hist.length - 1) $scope.nextHistoryButton = true;
            else $scope.nextHistoryButton = false;
        } else {
            $scope.backHistoryButton = false;
            $scope.nextHistoryButton = false;
        }
    }

    function openSiblingApp() {
        if($scope.sibling.openNow !== "undefined" && $scope.sibling.openNow) {
            $scope.data.hideOpenRemember = true;
            localStorage.setItem("openSiblingNow", true);
        }
        if(typeof $scope.data.dnvPkey != "undefined") {
            openSibling.openPkey($scope.data.dnvPkey, $scope.data.dnvAcc);
        } else {
            openSibling.openWord($scope.data.word);
        }
    }
})

.controller('AdvancedSearchController', function($state, $scope, $rootScope, $stateParams, $ionicLoading, $ionicPopup, api, historial) {

    /*
     * - texto: word
     * - co_criteri: radio criteria (1: texto%, 2: %texto%, 3:%texto)
     * - co_tema: àrea temàtica
     * - co_catgram: combo categoria gramatical
     * - co_llengua: combo nivell lenguatge
     * - co_origen: combo llengua origen
     * - co_catsem: combo intenció parlant
     * - in_definicio: input definicio
     * - in_exemple: input exemple
     */

    $scope.data = {
        co_criteri: "2"
    };

    if (typeof $rootScope.advancedSearch != "undefined" && $rootScope.advancedSearch != null) {
        $scope.data = $rootScope.advancedSearch;
    }

    if (typeof $stateParams.word != "undefined" && $stateParams.word != null) {
        $scope.data.texto = $stateParams.word;
    }

    $scope.infoDropDowns = $rootScope.infoDropDowns;

    $scope.submit = function() {
        var info = {};
        var filtres = {};

        if (isNotEmpty($scope.data.texto)) {
            info.texto = $scope.data.texto;
        } else {
            $scope.data.texto = "";
        }
        if (isNotEmpty($scope.data.co_tema)) {
            info.co_tema = $scope.data.co_tema;
            filtres.area = $scope.data.co_tema;
        }
        if (isNotEmpty($scope.data.co_catgram)) {
            info.co_catgram = $scope.data.co_catgram;
            filtres.categoria = $scope.data.co_catgram;
        }
        if (isNotEmpty($scope.data.co_origen)) {
            info.co_origen = $scope.data.co_origen;
            filtres.llengua = $scope.data.co_origen;
        }
        if (isNotEmpty($scope.data.in_definicio)) {
            info.in_definicio = $scope.data.in_definicio;
            filtres.definicio = $scope.data.in_definicio;
        }

        if (angular.equals({}, info)) {
            var alertPopup = $ionicPopup.alert({
                title: "Busca buida",
                template: "Cal especificar una consulta.",
                buttons: [{ text: "Tancar" }]
            });
        } else {
            if (isNotEmpty($scope.data.co_criteri)) {
                info.co_criteri = $scope.data.co_criteri;
                switch (info.co_criteri) {
                    case "1":
                        filtres.criteris = "Comença";
                        break;
                    case "2":
                        filtres.criteris = "Conté";
                        break;
                    case "3":
                        filtres.criteris = "Acaba";
                        break;
                }
            }

            api.log({
                busqueda: $scope.data.texto,
                navegador: ionic.Platform.platform(),
                filtres: filtres,
                seleccionada: ""
            });

            $rootScope.advancedSearch = info;
            $state.go("app.advanced-search-results");
        }
    }
})

.controller('AdvancedSearchResultsController', function($state, $scope, $rootScope, api) {

    $scope.page = 0;
    $scope.moredata = true;
    $scope.items = [];
    $scope.data = $rootScope.advancedSearch

    //Infinite Scroll Pagination
    $scope.loadMore = function() {
        loadMore();
    }

    $scope.openDefinition = function(term) {
        api.log({
            busqueda: term.lema,
            navegador: ionic.Platform.platform(),
            filtres: {},
            seleccionada: term.lema
        });
        $state.go("app.simple-search", {
            term: term,
            in_exemple: $rootScope.advancedSearch.in_exemple,
            in_definicio: $rootScope.advancedSearch.in_definicio,
            fromAdvance: true
        });
    }

    function loadMore() {
        api.advancedSearch($scope.page * 20, 20, $scope.data).then(function(result) {
            $scope.items = $scope.items.concat(result.data);
            if (result.data.length == 0 || result.data.length < 20) $scope.moredata = false;
            else $scope.page++;
            $scope.$broadcast('scroll.infiniteScrollComplete');
        });
    }

});

function isNotEmpty(param) {
    return (typeof param != "undefined" && param != null && param != "")
}